//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Globalization;
using System.Windows.Forms;

using MindFusion.Drawing;


namespace MindFusion.Scheduling.WinForms.Samples.CS.FullYear
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			calendar.ListViewSettings.FreeDrag = State.Enabled;
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void calendar_Draw(object sender, DrawEventArgs e)
		{
			Painter painter = new Painter(e.Graphics);

			DateTime date = e.Date;
			if (e.Element == CustomDrawElements.CellHeader ||
				e.Element == CustomDrawElements.CellContents)
			{
				Region old = e.Graphics.Clip;
				Region @new = new Region(e.Bounds);
				e.Graphics.Clip = @new;

				int size = calendar.ItemSettings.Size + calendar.ItemSettings.Spacing;
				int y = e.Bounds.Top;
				int i = 0;
				while (y < e.Bounds.Bottom)
				{
					Rectangle current = new Rectangle(e.Bounds.Left, y, e.Bounds.Width, size);

					if (e.Element == CustomDrawElements.CellHeader)
					{
						painter.FillHeaderBackground(current, calendar.ListViewSettings.Style);
						painter.DrawHeaderBorder(current, calendar.ListViewSettings.Style);

						DateTime d = date.Add(TimeSpan.FromDays(i));
						string dayOfWeek = calendar.DateTimeFormat.AbbreviatedDayNames[(int)d.DayOfWeek].Substring(0, 1);
						painter.DrawHeaderText(string.Format("{0}, {1}", dayOfWeek, d.Day), current, e.Style);
					}
					else
					{
						painter.DrawBorderBottomLine(current, e.Style, false, false);
					}

					y += size;
					i++;
					if (i >= DateTime.DaysInMonth(date.Year, date.Month))
						break;
				}

				e.Graphics.Clip = old;
				@new.Dispose();
			}
		}

		private void calendar_ItemModifying(object sender, ItemModifyConfirmEventArgs e)
		{
			// Check if the item leaves the month
			if (e.NewListLane > DateTime.DaysInMonth(e.NewStartTime.Year, e.NewStartTime.Month))
				e.Confirm = false;
		}

		private void calendar_ItemCreated(object sender, ItemEventArgs e)
		{
			e.Item.EndTime = e.Item.StartTime;
			e.Item.AllowChangeStart = false;
			e.Item.AllowChangeEnd = false;
		}


		private string _appPath;
	}
}